/*
  --- Program  : ScriptVariable.cpp
  --- Language : C++
  --- Notes    : Support class for accessing script variables created by
               : XIVR v 2.50
  --- Copyright: Xtend Technologies (P) Ltd.  grants  all legal XIVR users
               : the right to use, modify or include the following source
               : code in your programs.
*/

#include "stdafx.h"
#include "ScriptVariable.h"

/////////////////////////////////////////////////////////////////////////////
//
CScriptVariable::CScriptVariable()
{
	pFile = NULL;
	NoofLines = -1;
}

CScriptVariable::~CScriptVariable()
{
	Close();
}


BOOL CScriptVariable::Open(string filename)
{
	pFile = fopen(filename.c_str(),"rb");
	if(! pFile)
		return false;

	ScriptFile = filename;
	return LoadFileToArray();
}

void CScriptVariable::Reset(const char* pVar)
{
	string var_name;
	string var_value;

	string lhsvar = pVar;
	if( *lhsvar.begin() != '$')
		lhsvar = "$" + lhsvar;

	for(STRING2STRING::iterator it = theMap.begin();it != theMap.end();++it) {
		var_name = it->first;
		char* pTemp = (char*) var_name.c_str();

		if(strnicmp(pTemp,lhsvar.c_str(),lhsvar.length()) == 0)
			SetString(var_name,"");
	}
	return;
}


void CScriptVariable::Close()
{
	if(pFile == NULL)
		return;

	pFile = fopen(ScriptFile.c_str(),"wb");
	if(! pFile)
		return;

	string var_name;
	string var_value;
	for(STRING2STRING::iterator it = theMap.begin();it != theMap.end();++it) {
		var_name = it->first;
		var_value = Text2Hex(theMap[var_name]);
		fprintf(pFile,"%s %s\r\n",var_name.c_str(),var_value.c_str());
	}

	fclose(pFile);
	pFile = NULL;


	// Deallocate dynamic memory blocks
	theMap.clear();
	return;
}

long CScriptVariable::GetNum(string variable)
{
	string var_name = _strlwr((char*)variable.c_str());
	string var_value = theMap[var_name];
	int varval;
	varval = atoi(var_value.c_str());
	return varval;
}

void CScriptVariable::SetNum(string variable, long iValue)
{
	string var_name = _strlwr((char*)variable.c_str());
	char num[255];

	sprintf(num,"%d",iValue);
	theMap[var_name] = num;
}

string CScriptVariable::GetString(string variable)
{
	string var_name = _strlwr((char*)variable.c_str());
	string var_value;

	var_value = theMap[var_name];
	return var_value;
}

void CScriptVariable::SetString(string variable, string value)
{
	string var_name = _strlwr((char*)variable.c_str());
	theMap[var_name] = value;
}



// Implementation of internal functions
//---------------------------------------
bool CScriptVariable::LoadFileToArray ()
{
	size_t fsize;
	char *pBuf;
	
	// determin the file size
	//-----------------------
	if(fseek(pFile,0,SEEK_END) != 0)
		return false;
	fsize = ftell(pFile);
	rewind(pFile);
	
	// allocate a buffer for reading the full file
	//--------------------------------------------
	pBuf =(char*) malloc(fsize+1);
	if(pBuf == NULL)
		return false;
	
	if(fread(pBuf,fsize,1,pFile) != 1)	{
		free(pBuf);
		return false;
	}
	pBuf[fsize] = '\0';
	
	
	//Okay now we have all the file contents in memory, so create link list
	//---------------------------------------------------------------------
	char *ptr = pBuf;
	char *end = pBuf + fsize;
	char *curr;
	
	while (ptr < end) 	{
		if (NoofLines != -1) {
			if (NoofLines == 0)
				break;
			--NoofLines;
		}
		
		//Parse one line
		//--------------
		curr = ptr;
		while (ptr < end && *ptr != '\n' && *ptr != 26)
			++ptr;
		
		if (*ptr == '\n' || *ptr == 26) {
			*ptr = '\0';
			if ((ptr-1) >= curr && *(ptr-1) == '\r')
				*(ptr-1) = '\0';
		}
		ParseLine(string(curr));
		++ptr;
	}
	free(pBuf);
	fclose(pFile);
	return true;
}

string CScriptVariable::Text2Hex(string str)
{
	char hex[10];
	string ret;
	char *pStr = (char *) str.c_str();
	
	for (;*pStr; pStr++) {
		if (*pStr < 32 || *pStr > 126) {
			sprintf(hex,"%%%02x\0", (int)(char )*pStr);
			ret = ret + hex;
		}
		else {
			ret = ret + ((char) (*pStr));
		}
	}
	return ret;
}

string CScriptVariable::Hex2Text(string str)
{
	int val;
	string text;
	char *pStr = (char*) str.c_str();

	for (;*pStr; ++pStr) {
		if (*pStr == '%') {
			if (!*(pStr+1)) {
				text += (char) (*pStr);
				continue;
			}
			if (!*(pStr+2)) {
				text += (char) (*pStr);
				continue;
			}
			sscanf(pStr+1, "%02x", &val);
			text += (char) val;
			pStr += 2;
		}
		else {
			text += (char) (*pStr);
		}
	}
	return text;
}

void CScriptVariable::ParseLine(string str)
{
	int index;
	string var_name;
	string var_value;
	
	index = str.find(" ");
	if(index == -1)	{
		theMap[str] = "";
		return;
	}

	var_name = _strlwr((char*)str.substr(0,index).c_str());
	var_value = str.substr(index+1);
	theMap[var_name] = var_value;
}

